@@ -23,12 +23,12 @@ This encapsulates all services of the backend and manages all handlers.
 
 import logging, os, os.path, signal, threading, sys
 
-import gobject
 import dbus
 import dbus.service
 import dbus.mainloop.glib
+from gi.repository import GLib
 
-from oslib import OSLib
+from jockey.oslib import OSLib
 import detection, xorg_driver
 
 DBUS_BUS_NAME = 'com.ubuntu.DeviceDriver'
@@ -75,7 +75,7 @@ def dbus_sync_call_signal_wrapper(dbus_iface, fn, handler_map, *args, **kwargs):
         _h_exception_exc = exception
         loop.quit()
 
-    loop = gobject.MainLoop()
+    loop = GLib.MainLoop()
     global _h_reply_result, _h_exception_exc
     _h_reply_result = None
     _h_exception_exc = None
@@ -357,6 +357,8 @@ class Backend(dbus.service.Object):
                     self.remove_progress(-1, -1)
         if self._f_exception:
             raise self._f_exception[0], self._f_exception[1], self._f_exception[2]
+            # with py3:
+            #raise self._f_exception[0](self._f_exception[1]).with_traceback(self._f_exception[2])
 
         # notify about reboot
         if not self._f_result:
@@ -663,13 +665,13 @@ class Backend(dbus.service.Object):
         once the server is ready to take requests.
         '''
         dbus.service.Object.__init__(self, self.bus, '/DeviceDriver')
-        self.main_loop = gobject.MainLoop()
+        self.main_loop = GLib.MainLoop()
         self._timeout = False
         if timeout:
             def _t():
                 self.main_loop.quit()
                 return True
-            gobject.timeout_add(timeout * 1000, _t)
+            GLib.timeout_add(timeout * 1000, _t)
 
         # send parent process a signal that we are ready now
         if send_usr1:
