@@ -24,13 +24,23 @@ unknown hardware.
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, os.path, subprocess, sys, logging, re, urllib2
-import cPickle, xmlrpclib, locale
+import os, os.path, subprocess, sys, logging, re, locale
 from glob import glob
-from cStringIO import StringIO
-import pycurl
+# prefer Python 2 module here, as in Python 2 io.StringIO is broken
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from io import StringIO
 
-from oslib import OSLib
+try:
+    from xmlrpc.client import ServerProxy
+    import pickle
+except ImportError:
+    import cPickle as pickle
+    from xmlrpclib import ServerProxy
+    import pycurl
+
+from jockey.oslib import OSLib
 import handlers, xorg_driver
 
 #--------------------------------------------------------------------#
@@ -129,6 +139,9 @@ class DriverID:
     def __setitem__(self, key, value):
         self.properties.__setitem__(key, value)
 
+    def __str__(self):
+        return str(self.properties)
+
 #--------------------------------------------------------------------#
 
 class DriverDB:
@@ -156,8 +169,8 @@ class DriverDB:
             self.cache_path = os.path.join(OSLib.inst.backup_dir,
                 'driverdb-%s.cache' % self._cache_id())
             try:
-                self.cache = cPickle.load(open(self.cache_path, 'rb'))
-            except (cPickle.PickleError, IOError) as e:
+                self.cache = pickle.load(open(self.cache_path, 'rb'))
+            except (pickle.PickleError, IOError) as e:
                 logging.warning('Could not open DriverDB cache %s: %s',
                     self.cache_path, str(e))
                 self.cache = None
@@ -187,9 +200,9 @@ class DriverDB:
         if self.use_cache:
             try:
                 f = open(self.cache_path, 'wb')
-                cPickle.dump(self.cache, f)
+                pickle.dump(self.cache, f)
                 f.close()
-            except (cPickle.PickleError, IOError) as e:
+            except (pickle.PickleError, IOError) as e:
                 logging.warning('Could not create DriverDB cache %s: %s',
                     self.cache_path, str(e))
 
@@ -441,7 +454,7 @@ class XMLRPCDriverDB(DriverDB):
         '''Query remote server for driver updates for a set of HardwareIDs.'''
 
         logging.debug('Querying XML-RPC driver database %s...', self.url)
-        client = xmlrpclib.ServerProxy(self.url)
+        client = ServerProxy(self.url)
         # TODO: error handling; pass kernel_version, architecture
         (res_proto_ver, res_proto_subver, drivers) = client.query(
             '20080407', '0', { 
@@ -589,9 +602,8 @@ class OpenPrintingDriverDB(DriverDB):
                             s.get('level', 'voluntary'), s['url'])
                 did.properties['long_description'] = {'C': desc}
 
-                # needed to disambiguate e. g. splix and splix2
-                if driver != did.properties['package']:
-                    did.properties['version'] = driver.replace('driver/', '')
+                if 'version' in pkgs[pkg]:
+                    did.properties['version'] = pkgs[pkg]['version']
 
                 if fingerprint:
                     did.properties['fingerprint'] = fingerprint
@@ -723,7 +735,9 @@ def _driverid_to_handler(did, backend, mode):
             # win over autogenerated standard ones
             for h in backend.handler_pool.itervalues():
                 if isinstance(h, handlers.KernelModuleHandler) and \
-                    h.module == did['kernel_module']:
+                        h.module == did['kernel_module'] and \
+                        (not h.package or not did['package'] or h.package == did['package']):
+                    logging.debug('found match in handler pool %s', h)
                     del kwargs['kernel_module']
                     hclass = h.__class__
                     break
@@ -1011,6 +1025,7 @@ def get_db_handlers(backend, db, hardware, mode=MODE_ANY):
                 pass
 
         for did in dids:
+            logging.debug('searching handler for driver ID %s', str(did))
             h = _driverid_to_handler(did, backend, mode)
             if h:
                 if h.available() == False:
