@@ -26,12 +26,16 @@ class OSLib:
     # global default instance
     inst = None
 
-    def __init__(self, client_only=False):
+    def __init__(self, client_only=False, target_kernel=None):
         '''Set default paths and load the module blacklist.
         
         Distributors might want to override some default paths.
         If client_only is True, this only initializes functionality which is
         needed by clients, and which can be done without special privileges.
+
+        If target_kernel is given, it is used instead of the default
+        os.uname()[2]. This is primarily useful for distribution installers
+        where the target system kernel differs from the installer kernel.
         '''
         # relevant stuff for clients and backend
         self._get_os_version()
@@ -93,7 +97,19 @@ class OSLib:
         # this is the generally recommended DNS round-robin, but usually very
         # slow:
         #self.gpg_key_server = 'keys.gnupg.net'
-        self.gpg_key_server = 'keyserver.ubuntu.com'
+        self.gpg_key_server = 'hkp://keyserver.ubuntu.com:80'
+
+        if target_kernel:
+            self.target_kernel = target_kernel
+        else:
+            self.target_kernel = os.uname()[2]
+
+        # Package which provides include files for the currently running
+        # kernel.  If the system ensures that kernel headers are always
+        # available, or being pulled in via dependencies (and there are not
+        # multiple kernel flavors), it is ok to set this to "None". This should
+        # use self.target_kernel instead of os.uname()[2].
+        self.kernel_header_package = None
 
     # 
     # The following package related functions use PackageKit; if that does not
@@ -184,7 +200,8 @@ class OSLib:
         that the repository is signed with that key.
 
         An unknown package should raise a ValueError. Any installation failure
-        should be raised as a SystemError.
+        due to bad packages should be logged, but not raise an exception, as
+        this would just crash the backend.
         '''
         if repository or fingerprint:
             raise NotImplementedError('PackageKit default implementation does not currently support repositories or fingerprints')
@@ -221,7 +238,7 @@ class OSLib:
 
         err += pkcon.stderr.read()
         if pkcon.wait() != 0 or not self.package_installed(package):
-            raise SystemError('package %s failed to install: %s' % (package, err)) 
+            logging.error('package %s failed to install: %s' % (package, err))
             
     def remove_package(self, package, progress_cb):
         '''Uninstall the given package.
@@ -335,7 +352,8 @@ class OSLib:
                 '--no-default-keyring', '--primary-keyring', default_keyring,
                 '--keyserver', self.gpg_key_server, '--recv-key', keyid], 
                 stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
-                env={'PATH': os.environ.get('PATH', '')})
+                env={'PATH': os.environ.get('PATH', ''), 
+                     'http_proxy': os.environ.get('http_proxy', '')})
             (out, err) = gpg.communicate()
 
             if fingerprint not in self._gpg_keyring_fingerprints(default_keyring):
