@@ -22,19 +22,18 @@ functions with an appropriate toolkit.
 
 import gettext, optparse, urllib2, tempfile, sys, time, os, threading, signal
 
-import gobject
 import dbus
 import dbus.service
 import dbus.mainloop.glib
 
 try:
-    from gi.repository import AppIndicator
+    from gi.repository import AppIndicator3 as AppIndicator
 except:
     # UIs will need to implement their own status icon if app indicator isn't
     # available
     pass
 
-from oslib import OSLib
+from jockey.oslib import OSLib
 from backend import UnknownHandlerException, PermissionDeniedByPolicy, \
     BackendCrashError, convert_dbus_exceptions, \
     dbus_sync_call_signal_wrapper, Backend, DBUS_BUS_NAME
@@ -47,6 +46,23 @@ def bool(str):
     assert str == 'False'
     return False
 
+# Avoid having to do .encode('UTF-8') everywhere. This is a pain; I wish
+# Python supported something like "sys.stdout.encoding = 'UTF-8'".
+def fix_stdouterr():
+    import codecs
+    import locale
+    def null_decode(input, errors='strict'):
+        return input, len(input)
+    encoding = locale.getpreferredencoding()
+    # happens for e. g. LC_MESSAGES=*.UTF-8 LANG=C (forgetting to set LC_CTYPE
+    # as well); force UTF-8 in that case
+    if encoding == 'ANSI_X3.4-1968':
+        encoding = 'UTF-8'
+    sys.stdout = codecs.EncodedFile(sys.stdout, encoding)
+    sys.stdout.decode = null_decode
+    sys.stderr = codecs.EncodedFile(sys.stderr, encoding)
+    sys.stderr.decode = null_decode
+
 class AbstractUI(dbus.service.Object):
     '''Abstract user interface.
 
@@ -62,9 +78,11 @@ class AbstractUI(dbus.service.Object):
         gettext.install('jockey', unicode=True)
 
         (self.argv_options, self.argv_args) = self.parse_argv()
+        fix_stdouterr()
 
         if not OSLib.inst:
-            OSLib.inst = OSLib(client_only=not self.argv_options.no_dbus)
+            OSLib.inst = OSLib(client_only=not self.argv_options.no_dbus,
+                    target_kernel=self.argv_options.kernel)
 
         if self.argv_options.check:
             time.sleep(self.argv_options.check)
@@ -374,6 +392,9 @@ class AbstractUI(dbus.service.Object):
                 help=self._('Run as session D-BUS server.'))
         parser.add_option ('--no-dbus', action='store_true', default=False,
                 help=self._('Do not use D-BUS for communicating with the backend. Needs root privileges.'))
+        parser.add_option ('-k', '--kernel', type='string',
+                help=_('Use a different target kernel version than the currently running one. This is only relevant with --no-dbus.'))
+
         #parser.add_option ('--debug', action='store_true',
         #        dest='debug', default=False,
         #        help=self._('Enable debugging messages.'))
@@ -855,7 +876,8 @@ class AbstractUI(dbus.service.Object):
         dbus_name = dbus.service.BusName(DBUS_BUS_NAME, bus)
 
         dbus.service.Object.__init__(self, bus, '/GUI')
-        self.dbus_server_main_loop = gobject.MainLoop()
+        from gi.repository import GLib
+        self.dbus_server_main_loop = GLib.MainLoop()
         self.dbus_server_main_loop.run()
 
     @classmethod
