@@ -25,7 +25,7 @@ import jockey.detection, jockey.backend
 from jockey.detection import HardwareID
 from jockey.oslib import OSLib
 
-import sandbox, http
+import sandbox, httpd
 
 class DetectionTest(sandbox.LogTestCase):
     def tearDown(self):
@@ -443,6 +443,92 @@ class PickyHandler(jockey.handlers.KernelModuleHandler):
             for f in os.listdir(OSLib.inst.modaliases[1]):
                 os.unlink(os.path.join(OSLib.inst.modaliases[1], f))
 
+    def test_localkernelmod_driverdb_alternative_handler(self):
+        '''LocalKernelModulesDriverDB offers alternative handlers for the same module'''
+
+        # test handler which is unavailable for mesa-vanilla
+        open (os.path.join(OSLib.inst.handler_dir, 'testhandlers.py'), 'w').write('''
+import jockey.handlers
+
+class VanillaHandler(jockey.handlers.KernelModuleHandler):
+    def __init__(self, backend, package=None):
+        jockey.handlers.KernelModuleHandler.__init__(self, backend, 'chocolate')
+        self.package = package or 'mesa-vanilla'
+
+    def id(self):
+        return 'kmod:%s:%s' % (self.module, self.package)
+
+class VanillaUpdatesHandler(VanillaHandler):
+    def __init__(self, backend):
+        VanillaHandler.__init__(self, backend, 'mesa-vanilla-updates')
+''')
+
+        try:
+            f = open(os.path.join(OSLib.inst.modaliases[1], 'vanilla.alias'), 'w')
+            f.write('''alias pci:v0000FFF0d*1sv*sd*bc06sc*i* chocolate mesa-vanilla
+alias pci:v0000FFF0d*1sv*sd*bc06sc*i* chocolate mesa-vanilla-updates
+''')
+            f.close()
+
+            result = jockey.detection.get_handlers(jockey.backend.Backend(),
+                jockey.detection.LocalKernelModulesDriverDB())
+
+            out = '\n'.join(sorted([str(h) for h in result if h.module == 'chocolate']))
+
+            self.assertEqual(out, '''kmod:chocolate:mesa-vanilla([VanillaHandler, nonfree, disabled] free module with nonavailable hardware)
+kmod:chocolate:mesa-vanilla-updates([VanillaUpdatesHandler, nonfree, disabled] free module with nonavailable hardware)''')
+
+        finally:
+            for f in os.listdir(OSLib.inst.modaliases[1]):
+                os.unlink(os.path.join(OSLib.inst.modaliases[1], f))
+
+    def test_localkernelmod_driverdb_alternative_handler_xorg(self):
+        '''LocalKernelModulesDriverDB offers alternative X.org handlers for the same module'''
+
+        # test handler which is unavailable for mesa-vanilla
+        open (os.path.join(OSLib.inst.handler_dir, 'testhandlers.py'), 'w').write('''
+import jockey.handlers
+
+class VanillaHandler(jockey.xorg_driver.XorgDriverHandler):
+    def __init__(self, backend, package=None):
+        if not package:
+            package = 'mesa-vanilla'
+        if package and 'update' in package:
+            name = 'Vanilla 3D driver (post-release updates)'
+        else:
+            name = 'Vanilla 3D driver'
+        jockey.xorg_driver.XorgDriverHandler.__init__(self, backend,
+          'chocolate', package, 'vanilla3d', 'vanilla', 
+          name=name)
+
+class VanillaUpdatesHandler(VanillaHandler):
+    def __init__(self, backend):
+        VanillaHandler.__init__(self, backend, 'mesa-vanilla-updates')
+
+    def id(self):
+        return VanillaHandler.id(self) + '-updates'
+
+''')
+
+        try:
+            f = open(os.path.join(OSLib.inst.modaliases[1], 'vanilla.alias'), 'w')
+            f.write('''alias pci:v0000FFF0d*1sv*sd*bc06sc*i* chocolate mesa-vanilla
+alias pci:v0000FFF0d*1sv*sd*bc06sc*i* chocolate mesa-vanilla-updates
+''')
+            f.close()
+
+            result = jockey.detection.get_handlers(jockey.backend.Backend(),
+                jockey.detection.LocalKernelModulesDriverDB())
+
+            out = '\n'.join(sorted([str(h) for h in result if h.module == 'chocolate']))
+
+            self.assertEqual(out, '''xorg:chocolate([VanillaHandler, nonfree, disabled] Vanilla 3D driver)
+xorg:chocolate-updates([VanillaUpdatesHandler, nonfree, disabled] Vanilla 3D driver (post-release updates))''')
+
+        finally:
+            for f in os.listdir(OSLib.inst.modaliases[1]):
+                os.unlink(os.path.join(OSLib.inst.modaliases[1], f))
+
     def test_xmlrpc_driverdb_no_systemid(self):
         '''XMLRPCDriverDB with a working (local) server and no system ID'''
 
@@ -776,7 +862,7 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         finally:
             OSLib.inst.packaging_system = orig_pkgsys
         handlers = backend.available()
-        self.assertEqual(handlers, ['printer:openprinting-ppds-postscript-hp:Postscript-HP'])
+        self.assertEqual(handlers, ['printer:openprinting-ppds-postscript-hp:20091009'])
 
         hi = backend.handler_info(handlers[0])
         self.assertEqual(hi['package'], 'openprinting-ppds-postscript-hp')
@@ -799,7 +885,7 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         finally:
             OSLib.inst.packaging_system = orig_pkgsys
         handlers = backend.available()
-        self.assertEqual(handlers, ['printer:epson-inkjet-printer-n10-nx127:epson-n10-nx127'])
+        self.assertEqual(handlers, ['printer:epson-inkjet-printer-n10-nx127:1.0.0'])
 
         hi = backend.handler_info(handlers[0])
         self.assert_(hi['package'].startswith('epson-inkjet-'))
@@ -863,13 +949,13 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
         OSLib.inst.ssl_cert_file_paths = [self.localhost_pem]
 
     def tearDown(self):
-        http.stop()
+        httpd.stop()
         OSLib.inst.ssl_cert_file_paths = self.orig_cert_file_paths
 
     def test_valid(self):
         '''GPG fingerprint: valid certificate, hostname, and fingerprint'''
 
-        http.start(4443, self.data_dir, True, self.localhost_pem)
+        httpd.start(4443, self.data_dir, True, self.localhost_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://localhost:4443/fingerprint')
@@ -878,7 +964,7 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
     def test_bad_fingerprint_syntax(self):
         '''GPG fingerprint: no fingerprint found'''
 
-        http.start(4443, self.data_dir, True, self.localhost_pem)
+        httpd.start(4443, self.data_dir, True, self.localhost_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://localhost:4443/bad_fingerprint')
@@ -887,7 +973,7 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
     def test_404(self):
         '''GPG fingerprint: nonexisting address'''
 
-        http.start(4443, self.data_dir, True, self.localhost_pem)
+        httpd.start(4443, self.data_dir, True, self.localhost_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://localhost:4443/nonexisting')
@@ -896,7 +982,7 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
     def test_wrong_host(self):
         '''GPG fingerprint: unknown hostname'''
 
-        http.start(4443, self.data_dir, True, self.localhost_pem)
+        httpd.start(4443, self.data_dir, True, self.localhost_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://127.0.0.1:4443/fingerprint')
@@ -905,7 +991,7 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
     def test_expired_cert(self):
         '''expired certificate'''
 
-        http.start(4443, self.data_dir, True, self.localhost_expired_pem)
+        httpd.start(4443, self.data_dir, True, self.localhost_expired_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://127.0.0.1:4443/fingerprint')
@@ -915,7 +1001,7 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
         '''wrong certificate'''
 
         OSLib.inst.ssl_cert_file_paths = [self.localhost_expired_pem]
-        http.start(4443, '.', True, self.localhost_pem)
+        httpd.start(4443, '.', True, self.localhost_pem)
         result = jockey.detection.download_gpg_fingerprint(
                 'https://127.0.0.1:4443/fingerprint')
         self.assertEqual(result, None, result)
@@ -924,11 +1010,11 @@ class GPGFingerPrintTest(sandbox.LogTestCase):
         '''system-wide trusted certificate shouldn't match local test server'''
 
         if not self.orig_cert_file_paths:
-            print '[SKIP, not available]'
+            print('[SKIP, not available]')
             return
 
         OSLib.inst.ssl_cert_file_paths = self.orig_cert_file_paths
-        http.start(4443, '.', True, self.localhost_pem)
+        httpd.start(4443, '.', True, self.localhost_pem)
 
         result = jockey.detection.download_gpg_fingerprint(
                 'https://127.0.0.1:4443/fingerprint')
