@@ -234,6 +234,7 @@ class HandlerTest(sandbox.LogTestCase):
         self.assert_(h.changed())
         self.assert_(h.enabled())
         self.assert_(OSLib.inst.package_installed('mesa-vanilla')) 
+        self.failIf(OSLib.inst.package_installed('linux-dev')) 
         self.failIf(OSLib.inst.module_blacklisted('vanilla3d'))
 
         h.disable()
@@ -250,6 +251,22 @@ class HandlerTest(sandbox.LogTestCase):
         h.package = 'nonexisting'
         self.assertEqual(h.available(), False)
 
+    def test_kmod_package_kernelheaders(self):
+        '''KernelModuleHandler: package needing kernel headers'''
+
+        self.failIf(OSLib.inst.package_installed('mesa-vanilla')) 
+        h = jockey.handlers.KernelModuleHandler(self.backend,
+            'vanilla3d', rationale='Get full graphics acceleration speed')
+        h.package = 'mesa-vanilla'
+        h.needs_kernel_headers = True
+
+        h.enable()
+        self.assert_(h.changed())
+        self.assert_(h.enabled())
+        self.assert_(OSLib.inst.package_installed('mesa-vanilla')) 
+        self.assert_(OSLib.inst.package_installed('linux-dev')) 
+        self.failIf(OSLib.inst.module_blacklisted('vanilla3d'))
+
     def test_kmod_package_used(self):
         '''KernelModuleHandler: used() with third-party packages'''
 
@@ -339,7 +356,7 @@ class HandlerTest(sandbox.LogTestCase):
         # add available handler
         self.assertEqual(h.available(), None)
         names = {}
-        exec sandbox.h_availability_py in names
+        exec(sandbox.h_availability_py, names)
         h.add (names['AvailMod'](self.backend))
         self.assertEqual(h.available(), True)
 
@@ -351,7 +368,7 @@ class HandlerTest(sandbox.LogTestCase):
         # add unchangeable handler
         self.assertEqual(h.can_change(), None)
         names = {}
-        exec 'import jockey.handlers\n' + sandbox.h_nochangemod in names
+        exec('import jockey.handlers\n' + sandbox.h_nochangemod, names)
         h.add (names['NoChangeMod'](self.backend))
         self.assertNotEqual(h.can_change(), None)
 
@@ -869,6 +886,72 @@ EndSection
         self.assert_(OSLib.inst.module_blacklisted('vanilla3d'))
         self.failIf(os.path.exists(OSLib.inst.xorg_conf_path))
 
+    def test_xorg_driver_no_xorgconf_nodriver(self):
+        '''XorgDriverHandler without a default xorg.conf and no explicit "Driver"'''
+
+        os.unlink(OSLib.inst.xorg_conf_path)
+
+        h = XorgDriverHandler(self.backend, 'vanilla3d', 'mesa-vanilla', None,
+            None, extra_conf_options={'SuperSpeed': 'true'}, 
+            add_modules=['glx'], disable_modules=['dri2'],
+            name='Vanilla accelerated graphics driver')
+
+        self.failIf(h.free())
+        self.failIf(h.changed())
+        self.assertEqual(h.can_change(), None) # should create an xorg.conf from scratch
+        self.failIf(h.enabled())
+        self.assertEqual(h.available(), None)
+
+        self.failIf(h.enable())
+        self.assert_(h.changed())
+        self.assert_(h.enabled())
+        self.assert_(OSLib.inst.package_installed('mesa-vanilla')) 
+        self.failIf(OSLib.inst.module_blacklisted('vanilla3d'))
+
+        conf = open(OSLib.inst.xorg_conf_path).read()
+        self.failIf(re.search('^\s*driver', conf, re.I|re.M))
+        self.assert_(re.search('^\s*option\s*"SuperSpeed"\s*"true"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*load\s*"glx"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*disable\s*"dri2"\s*$', conf, re.I|re.M))
+        self.failIf(re.search('^\s*load\s*"dri"\s*$', conf, re.I|re.M))
+
+        # should remove xorg.conf again
+        self.failIf(h.disable())
+        self.assert_(h.changed())
+        self.failIf(h.enabled())
+        self.failIf(OSLib.inst.package_installed('mesa-vanilla')) 
+        self.assert_(OSLib.inst.module_blacklisted('vanilla3d'))
+        self.failIf(os.path.exists(OSLib.inst.xorg_conf_path))
+
+    def test_xorg_driver_nobackup_nodriver(self):
+        '''XorgDriverHandler with no xorg.conf backup and no explicit "Driver"'''
+        
+        h = XorgDriverHandler(self.backend, 'vanilla3d', 'mesa-vanilla', None,
+            None, extra_conf_options={'SuperSpeed': 'true'}, 
+            add_modules=['glx'], disable_modules=['dri2'], remove_modules=['dri'],
+            name='Vanilla accelerated graphics driver')
+
+        self.failIf(h.enable())
+        conf = open(OSLib.inst.xorg_conf_path).read()
+        # should keep the "Driver" as it was before
+        self.assert_(re.search('^\s*driver\s*"vanilla"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*option\s*"SuperSpeed"\s*"true"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*load\s*"glx"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*disable\s*"dri2"\s*$', conf, re.I|re.M))
+        self.failIf(re.search('^\s*load\s*"dri"\s*$', conf, re.I|re.M))
+
+        # unlink backup file
+        os.unlink(os.path.join(OSLib.inst.backup_dir, 'mesa-vanilla.oldconf'))
+
+        self.failIf(h.disable())
+        conf = open(OSLib.inst.xorg_conf_path).read()
+        self.assert_(re.search('^\s*driver\s*"vanilla"\s*$', conf, re.I|re.M))
+        self.failIf('SuperSpeed' in conf)
+        self.failIf(re.search('^\s*load\s*"glx"\s*$', conf, re.I|re.M))
+        # modules which were explicitly disabled should be removed
+        self.failIf(re.search('^\s*disable\s*"dri2"\s*$', conf, re.I|re.M))
+        self.assert_(re.search('^\s*load\s*"dri"\s*$', conf, re.I|re.M))
+    
     def test_xorg_invalid_conf(self):
         '''XorgDriverHandler with invalid xorg.conf'''
 
