@@ -21,8 +21,13 @@ for self tests.'''
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import tempfile, atexit, shutil, os, os.path, sys, signal, subprocess
-import unittest, xmlrpclib, time, socket, traceback
-from SimpleXMLRPCServer import SimpleXMLRPCServer
+import unittest, time, socket, traceback
+try:
+    from xmlrpc.client import ServerProxy
+    from xmlrpc.server import SimpleXMLRPCServer
+except ImportError:
+    from xmlrpclib import ServerProxy
+    from SimpleXMLRPCServer import SimpleXMLRPCServer
 
 from jockey.oslib import OSLib
 from jockey.detection import HardwareID, DriverID, DriverDB
@@ -236,6 +241,13 @@ fake_pkginfo = {
             'free': False,
             'files': ['/bin/mesa-vanilla-test', '/lib/X11/vesa.drv'],
         },
+        'mesa-vanilla-updates': {
+            'description': ('X.org libraries for the Vanilla 3D driver (updates)', 
+                'This package provides optimized Mesa libraries for the Vanilla '
+                'graphics cards.'),
+            'free': False,
+            'files': ['/bin/mesa-vanilla-test', '/lib/X11/vesa.drv'],
+        },
         'mesa-std': {
             'description': ('standard system mesa libraries',
                 'default mesa libs for free drivers'),
@@ -251,6 +263,11 @@ fake_pkginfo = {
                 'This is yummy, but uninteresting.'),
             'free': False,
         },
+        'linux-dev': {
+            'description': ('fake kernel headers',
+                'includes for building kernel modules'),
+            'free': True,
+        },
     },
     'http://vaportech.com/x1': {
         'spam-x1': {
@@ -270,7 +287,7 @@ class LogTestCase(unittest.TestCase):
     '''TestCase which logs method names to sandbox.log.'''
 
     def run(self, result=None):
-        print >> log, '--- Running test', self.id()
+        log.write('--- Running test %s\n' % self.id())
         unittest.TestCase.run(self, result)
 
 #-------------------------------------------------------------------#
@@ -307,6 +324,8 @@ class TestOSLib(OSLib):
         self.help_available = False
         self.help_called = False
 
+        self.kernel_header_package = 'linux-dev'
+
     def reset_packages(self):
         '''Reset repositories and packages.'''
 
@@ -458,7 +477,7 @@ else:
 
         f = open(self.proc_modules, 'w')
         for m in mods:
-            print >> f, '%s 12345 0 - Live 0xdeadbeef' % m
+            f.write('%s 12345 0 - Live 0xdeadbeef\n' % m)
         f.close()
 
     def _make_modalias(self):
@@ -475,14 +494,14 @@ else:
         os.mkdir(self.modaliases[1])
 
         f = open(self.modaliases[0], 'w')
-        print >> f, '# Aliases extracted from modules themselves.'
+        f.write('# Aliases extracted from modules themselves.\n')
         for mod in sorted(fake_modinfo.keys()):
             aliases = fake_modinfo[mod].get('alias', [])
             if hasattr(aliases, 'isspace'):
-                print >> f, 'alias %s %s' % (aliases, mod)
+                f.write('alias %s %s\n' % (aliases, mod))
             else:
                 for a in aliases:
-                    print >> f, 'alias %s %s' % (a, mod)
+                    f.write('alias %s %s\n' % (a, mod))
         f.close()
 
     def _make_xorg_conf(self):
@@ -605,7 +624,8 @@ EndSection
         that the repository is signed with that key.
 
         An unknown package should raise a ValueError. Any installation failure
-        should be raised as a SystemError.
+        due to bad packages should be logged, but not raise an exception, as
+        this would just crash the backend.
         '''
         if repository:
             if repository not in fake_pkginfo:
@@ -1002,12 +1022,12 @@ def start_driverdb_server(correct_protocol=True):
                 s.register_function(xmlrpc_driverdb_query_bogus, 'query')
             s.serve_forever()
         except:
-            print >> sys.stderr, '********** DEMO XML-RPC server failed: ***********'
+            sys.stderr.write('********** DEMO XML-RPC server failed: ***********\n')
             traceback.print_exc()
             os._exit(1)
 
     # wait until it is ready
-    c = xmlrpclib.ServerProxy('http://localhost:8080')
+    c = ServerProxy('http://localhost:8080')
     timeout = 100
     while timeout > 0:
         time.sleep(0.1)
@@ -1198,7 +1218,7 @@ alias pci:v00001001d00003D*sv*sd*bc03sc*i* vanilla3d
 class TestXMLRPCDriverDBConsistency(unittest.TestCase):
     def setUp(self):
         start_driverdb_server()
-        self.xmlrpc_client = xmlrpclib.ServerProxy('http://localhost:8080')
+        self.xmlrpc_client = ServerProxy('http://localhost:8080')
 
     def tearDown(self):
         stop_driverdb_server()
